/*************************************************************************
 *  Project Name:    FFT
 *  FileName:        fft.h
 *  Dependencies:    xc.h, stdint.h
 *  Processor:       PIC24, dsPIC
 *  Compiler:        XC16
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description: Declarations for the FFT library functions.
 *
 * written by Anton Alkhimenok 04/02/2015
 *************************************************************************/

#pragma once

#include <xc.h>
#include <stdint.h>

/*************************************************************************
 This parameter defines number of points/samples for the calculation.
 The valid values are 8, 16, 32, 64, 128, 256, 512, 1024 and 2048. 
 If required transform size doesnt match exactly the valid sizes
 then the size should be rounded to largest valid size and
 not used points should be set to zero.
 *************************************************************************/
#define FFT_POINTS_NUMBER      64

/*************************************************************************
 Function:

 void FFT(int16_t* pRe, int16_t* pIm)

 Description:
 
 This function calculates FFT for the points provided in real and imaginary parts arrays.
 In most cases the input signal is real (not complex), the data points must be stored in
 the real part array and the imaginary array elements must be set to zero.
 The sizes of the real and imaginary arrays must be equal to the transform size defined
 by FFT_POINTS_NUMBER in fft.h file. After calculation the result (harmonics) are written
 to the same input data arrays.

 Parameters:

 int16_t* re  pointer to array with the input signal vector (real part).
 int16_t* im  pointer to array with the input signal vector (imaginary part),
             for real signals this vector values must be set to zero.

 Returned data:

 int16_t* re  pointer to array where the harmonics are stored (real part).
 int16_t* im  pointer to array where the harmonics are stored (imaginary part).
*************************************************************************/
void FFT(int16_t* pRe, int16_t* pIm);

/*************************************************************************
 Function:

 uint16_t FFTAmplitude(int16_t re, int16_t im)

 Description:
 
 This function calculates absolute value (harmonic amplitude) of
 the complex number (A = SQRT(Re^2 + Im^2)).

 Parameters:

 int16_t re  real part of the complex number.
 int16_t im imaginary part of the complex number.

 Returned data:

 The function returns the absolute value (harmonic amplitude)
 for the complex number entered.
*************************************************************************/
uint16_t FFTAmplitude(int16_t re, int16_t im);

/*************************************************************************
 Function:

 int16_t FFTPhase(int16_t re, int16_t im);

 Description:
 
 This function calculates an argument (harmonic phase) of the complex number
 (D = (180/PI)*ARCTAN(Im/Re)) in degrees with maximum error +-1 degree.
 
 Parameters:

 short re  real part of the complex number.
 short im  imaginary part of the complex number.

 Returned data:
 The function returns an argument (harmonic phase) in degrees
 for the complex number entered.
*************************************************************************/
int16_t FFTPhase(int16_t re, int16_t im);
